@@ -1,6 +1,6 @@
 /* common.c - utility functions for all builtins */
 
-/* Copyright (C) 1987-2017 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2021 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -266,6 +266,13 @@ sh_readonly (s)
   builtin_error (_("%s: readonly variable"), s);
 }
 
+void
+sh_noassign (s)
+     const char *s;
+{
+  internal_error (_("%s: cannot assign"), s);	/* XXX */
+}
+
 void
 sh_erange (s, desc)
      char *s, *desc;
@@ -373,7 +380,8 @@ make_builtin_argv (list, ip)
 
 /* Remember LIST in $1 ... $9, and REST_OF_ARGS.  If DESTRUCTIVE is
    non-zero, then discard whatever the existing arguments are, else
-   only discard the ones that are to be replaced. */
+   only discard the ones that are to be replaced.  Set POSPARAM_COUNT
+   to the number of args assigned (length of LIST). */
 void
 remember_args (list, destructive)
      WORD_LIST *list;
@@ -381,6 +389,8 @@ remember_args (list, destructive)
 {
   register int i;
 
+  posparam_count = 0;
+
   for (i = 1; i < 10; i++)
     {
       if ((destructive || list) && dollar_vars[i])
@@ -391,7 +401,7 @@ remember_args (list, destructive)
 
       if (list)
 	{
-	  dollar_vars[i] = savestring (list->word->word);
+	  dollar_vars[posparam_count = i] = savestring (list->word->word);
 	  list = list->next;
 	}
     }
@@ -403,6 +413,7 @@ remember_args (list, destructive)
     {
       dispose_words (rest_of_args);
       rest_of_args = copy_word_list (list);
+      posparam_count += list_length (list);
     }
 
   if (destructive)
@@ -411,6 +422,58 @@ remember_args (list, destructive)
   invalidate_cached_quoted_dollar_at ();
 }
 
+void
+shift_args (times)
+     int times;
+{
+  WORD_LIST *temp;
+  int count;
+
+  if (times <= 0)		/* caller should check */
+    return;
+
+  while (times-- > 0)
+    {
+      if (dollar_vars[1])
+	free (dollar_vars[1]);
+
+      for (count = 1; count < 9; count++)
+	dollar_vars[count] = dollar_vars[count + 1];
+
+      if (rest_of_args)
+	{
+	  temp = rest_of_args;
+	  dollar_vars[9] = savestring (temp->word->word);
+	  rest_of_args = rest_of_args->next;
+	  temp->next = (WORD_LIST *)NULL;
+	  dispose_words (temp);
+	}
+      else
+	dollar_vars[9] = (char *)NULL;
+
+      posparam_count--;
+    }
+}
+
+int
+number_of_args ()
+{
+#if 0
+  register WORD_LIST *list;
+  int n;
+
+  for (n = 0; n < 9 && dollar_vars[n+1]; n++)
+    ;
+  for (list = rest_of_args; list; list = list->next)
+    n++;
+
+if (n != posparam_count)
+  itrace("number_of_args: n (%d) != posparam_count (%d)", n, posparam_count);
+#else
+  return posparam_count;
+#endif
+}
+
 static int changed_dollar_vars;
 
 /* Have the dollar variables been reset to new values since we last
@@ -678,7 +741,7 @@ get_job_spec (list)
   if (DIGIT (*word) && all_digits (word))
     {
       job = atoi (word);
-      return (job > js.j_jobslots ? NO_JOB : job - 1);
+      return ((job < 0 || job > js.j_jobslots) ? NO_JOB : job - 1);
     }
 
   jflags = 0;
@@ -879,7 +942,7 @@ find_special_builtin (name)
   			current_builtin->function :
   			(sh_builtin_func_t *)NULL);
 }
-  
+
 static int
 shell_builtin_compare (sbp1, sbp2)
      struct builtin *sbp1, *sbp2;
@@ -908,3 +971,127 @@ builtin_help ()
   printf ("%s: %s\n", this_command_name, _("help not available in this version"));
 }
 #endif
+
+/* **************************************************************** */
+/*								    */
+/*	    Variable assignments during builtin commands	    */
+/*								    */
+/* **************************************************************** */
+
+/* Assign NAME=VALUE, passing FLAGS to the assignment functions. */
+SHELL_VAR *
+builtin_bind_variable (name, value, flags)
+     char *name;
+     char *value;
+     int flags;
+{
+  SHELL_VAR *v;
+  int vflags, bindflags;
+
+#if defined (ARRAY_VARS)
+  /* Callers are responsible for calling this with array references that have
+     already undergone valid_array_reference checks (read, printf). */
+  vflags = assoc_expand_once ? (VA_NOEXPAND|VA_ONEWORD) : 0;
+  bindflags = flags | (assoc_expand_once ? ASS_NOEXPAND : 0) | ASS_ALLOWALLSUB;
+  if (flags & ASS_NOEXPAND)
+    vflags |= VA_NOEXPAND;
+  if (flags & ASS_ONEWORD)
+    vflags |= VA_ONEWORD;
+
+  if (valid_array_reference (name, vflags) == 0)
+    v = bind_variable (name, value, flags);
+  else
+    v = assign_array_element (name, value, bindflags, (array_eltstate_t *)0);
+#else /* !ARRAY_VARS */
+  v = bind_variable (name, value, flags);
+#endif /* !ARRAY_VARS */
+
+  if (v && readonly_p (v) == 0 && noassign_p (v) == 0)
+    VUNSETATTR (v, att_invisible);
+
+  return v;
+}
+
+SHELL_VAR *
+builtin_bind_var_to_int (name, val, flags)
+     char *name;
+     intmax_t val;
+     int flags;
+{
+  SHELL_VAR *v;
+
+  v = bind_var_to_int (name, val, flags|ASS_ALLOWALLSUB);
+  return v;
+}
+
+/* Like check_unbind_variable, but for use by builtins (only matters for
+   error messages). */
+int
+builtin_unbind_variable (vname)
+     const char *vname;
+{
+  SHELL_VAR *v;
+
+  v = find_variable (vname);
+  if (v && readonly_p (v))
+    {
+      builtin_error (_("%s: cannot unset: readonly %s"), vname, "variable");
+      return -2;
+    }
+  else if (v && non_unsettable_p (v))
+    {
+      builtin_error (_("%s: cannot unset"), vname);
+      return -2;
+    }
+  return (unbind_variable (vname));
+}
+
+int
+builtin_arrayref_flags (w, baseflags)
+     WORD_DESC *w;
+     int baseflags;
+{
+  char *t;
+  int vflags;
+
+  vflags = baseflags;
+
+  /* Don't require assoc_expand_once if we have an argument that's already
+     passed through valid_array_reference and been expanded once. That
+     doesn't protect it from normal expansions like word splitting, so
+     proper quoting is still required. */
+  if (w->flags & W_ARRAYREF)
+    vflags |= VA_ONEWORD|VA_NOEXPAND;
+
+#  if 0
+  /* This is a little sketchier but handles quoted arguments. */
+  if (assoc_expand_once && (t =  strchr (w->word, '[')) && t[strlen(t) - 1] == ']')
+    vflags |= VA_ONEWORD|VA_NOEXPAND;
+#  endif
+
+  return vflags;
+}
+
+/* **************************************************************** */
+/*								    */
+/*	    External interface to manipulate shell options	    */
+/*								    */
+/* **************************************************************** */
+
+#if defined (ARRAY_VARS)
+int
+set_expand_once (nval, uwp)
+     int nval, uwp;
+{
+  int oa;
+
+  oa = assoc_expand_once;
+  if (shell_compatibility_level > 51)	/* XXX - internal */
+    {
+      if (uwp)
+	unwind_protect_int (assoc_expand_once);
+      assoc_expand_once = nval;
+    }
+  return oa;
+}
+#endif
